from amaranth import Array, unsigned
from amaranth.lib import wiring, data
from amaranth.lib.wiring import Out


class RGBLayout(data.StructLayout):
    def __init__(self, r_bits, g_bits, b_bits):
        super().__init__(
            {
                "red": unsigned(r_bits),
                "green": unsigned(g_bits),
                "blue": unsigned(b_bits),
            }
        )


Rgb888Layout = RGBLayout(8, 8, 8)
Rgb666Layout = RGBLayout(6, 6, 6)

Rgb111Layout = RGBLayout(1, 1, 1)


class Hub75Stream(wiring.Signature):
    """A Hub75E Driver for a single string of panels."""

    def __init__(self):
        super().__init__(
            {
                "latch": Out(1),
                "oe": Out(1),
                "addr": Out(5),
                "rgb0": Out(3),
                "rgb1": Out(3),
                "display_clk": Out(1),
            }
        )


class Hub75Ctrl(wiring.Signature):
    """Hub75E control/addressing signals.
    This is separated because certain designs keep these common
    between all panel outputs (e.g Colorlight 5A-75b boards).

    """

    def __init__(self, n_strings: int = 1):
        super().__init__(
            {
                "latch": Out(1),
                "oe": Out(1),
                "addr": Out(5),
                "display_clk": Out(1),
                "data": Out(Hub75Data()).array(n_strings),
            }
        )


class Hub75Data(wiring.Signature):
    """Data lines for HUB75 displays. When combined with Hub75Ctrl, this forms a complete HUB75 interface.
    These are kept separate as some devices have a single set of control signals.
    """

    def __init__(self):
        super().__init__(
            {
                "rgb0": Out(Rgb111Layout),
                "rgb1": Out(Rgb111Layout),
            }
        )
